/*
 * MBSet.cpp
 *
 *  Created on: Dec 25, 2010
 *      Author: Tobias Brosch
 */

#include "MBSet.h"
#include <cassert>

MBSet::MBSet(int width, int height, QObject* parent) :
	QObject(parent), _params(width, height) {

}

void MBSet::updateModel() {
	_mat.resize(boost::extents[_params.getWidth()][_params.getHeight()]);
	double width = _params.getWidth();
	double height = _params.getHeight();
	for (int i = 0; i < width; ++i) {
		for (int j = 0; j < height; ++j) {
			double x = i;
			double y = j;
			_params.scr2World(x, y);
			_mat[i][j] = _params.get_num_iter(x, y);
		}
	}

	emit viewChanged(&_mat);
}

/*
 * MBSetParams:
 *
 *
 *
 *
 *
 *
 *
 */
MBSet::MBSetParams::MBSetParams(int width, int height) :
	_width(width), _height(height), _center_x(DEF_CENTER_X), _center_y(
			DEF_CENTER_Y), _scale(DEF_SCALE), _maxIter(500) {

}

void MBSet::MBSetParams::scr2World(double &x, double &y) const {
	x = (x - _width / 2) * _scale + _center_x;
	y = (_height / 2 - y) * _scale + _center_y;
}

double MBSet::MBSetParams::get_num_iter(double x_pos, double y_pos) {
	double x = 0;
	double y = 0;
	for (int i = 0; i < _maxIter; ++i) {
		double x_tmp = x * x - y * y + x_pos;
		y = 2 * x * y + y_pos;
		x = x_tmp;
		if (x * x + y * y > 32) {
			return std::pow((double) i / _maxIter, 0.2);
		}
	}
	return 0;
}

int MBSet::MBSetParams::getWidth() const {
	return _width;
}
int MBSet::MBSetParams::getHeight() const {
	return _height;
}
double MBSet::MBSetParams::getCenterX() const {
	return _center_x;
}
double MBSet::MBSetParams::getCenterY() const {
	return _center_y;
}
double MBSet::MBSetParams::getScale() const {
	return _scale;
}

int MBSet::MBSetParams::getMaxIter() const {
	return _maxIter;
}
